/**
* \file board.c
* \brief Plik z funkcjami zwiazanymi z plyta.
* \author Marek Sawicki
* \date 30-06-2011
*/

#include <stdio.h>
#include "board.h"
#include "player.h"
#include "uart.h"

uint32_t Volume;

void ioportInit (void);
void nvicConfiguration (void);
void extiConfiguration(void);

//---------------------------------------------------------------------
void boardInit (void){
   
   ioportInit();
   playerInit();
   uartsInit();

   printf("\r\n========== microWave board ===========\r\n");
//   printf("Compilation: %s,%s\r\n",__DATE__,__TIME__);
//   printf("============== Board init end ==========\r\n");

//    mc0 = ioc_getcb (NULL);
//    ioc_init (mc0);
//    ioc_read_info (&info, mc0);
//    
//    usbd_init();                          /* USB Initialization                 */
//    usbd_msc_init();
//    usbd_connect(__TRUE);                 /* USB Connect                        */
   sysTimerInit();
   nvicConfiguration();
}

void ioportInit (void){
	GPIO_InitTypeDef  GPIO_InitStructure;
   //Remap
   RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO, ENABLE); // Enable the Clock AFIO
   GPIO_PinRemapConfig(GPIO_Remap_SWJ_JTAGDisable,ENABLE);
   GPIO_PinRemapConfig(GPIO_PartialRemap_TIM3,ENABLE);
   
   // AMP cs 
   RCC_APB2PeriphClockCmd(csAMP_GPIO_CLK, ENABLE); // Enable the Clock 
   GPIO_InitStructure.GPIO_Pin = csAMP_GPIO_PIN;
   GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP; 
   GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
   AmpOff();
   GPIO_Init(csAMP_GPIO_PORT, &GPIO_InitStructure);
   
   // SD cs 
   RCC_APB2PeriphClockCmd(csSD_GPIO_CLK, ENABLE); // Enable the Clock 
   GPIO_InitStructure.GPIO_Pin = csSD_GPIO_PIN;
   GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP; 
   GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
   csSDSet();
   GPIO_Init(csSD_GPIO_PORT, &GPIO_InitStructure);

   // USB con 
   RCC_APB2PeriphClockCmd(conUSB_GPIO_CLK, ENABLE); // Enable the Clock 
   GPIO_InitStructure.GPIO_Pin = conUSB_GPIO_PIN;
   GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU; 
   GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
   GPIO_Init(conUSB_GPIO_PORT, &GPIO_InitStructure);

}

void nvicConfiguration (void){
   NVIC_InitTypeDef NVIC_InitStructure;

   NVIC_PriorityGroupConfig(NVIC_PriorityGroup_4);    // grupy 0,1,2,3,4
   // 16 priorytetow przerwan, 0-priorytet najwyzszy
   NVIC_SetPriority (SysTick_IRQn, 7);  // set Priority for Cortex-M0 System Interrupts
   
   // USART2 Interrupt 
   NVIC_InitStructure.NVIC_IRQChannel = USART2_IRQn;
   NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
   NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
   NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
   NVIC_Init(&NVIC_InitStructure);

   // IRQ TIM2 
   NVIC_InitStructure.NVIC_IRQChannel = TIM2_IRQn;
   NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 5;
   NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
   NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
   TIM_ClearITPendingBit(TIM2, TIM_FLAG_Update);
   NVIC_Init(&NVIC_InitStructure);
   
}

void extiConfiguration(void){
/*
EXTI_InitTypeDef EXTI_InitStructure;


  // Connect EXTI Line4 to PC.4 
  GPIO_EXTILineConfig(K1_GPIO_PORT_SOURCE, K1_GPIO_PIN_SOURCE);
  // Configure EXTI Lines to generate an interrupt    
  EXTI_InitStructure.EXTI_Line = EXTI_Line4;// | EXTI_Line7;
  EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
  //EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Event;
  //EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;
  //EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising; // wcisniecie
  EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising_Falling;
  EXTI_InitStructure.EXTI_LineCmd = ENABLE;
  EXTI_Init(&EXTI_InitStructure);
   */
}
