/**
* \file cmdRS.c
* \brief Plik z funkcjami interpretera komend RS232.
* \author Marek Sawicki
* \date 06-06-2012
*/
#include <stdio.h>
#include <string.h>
#include "ff.h"
#include "cmdRS.h"
#include "cmdHelp.h"
#include "wave.h"

DIR      dir;
FILINFO  fno;
FRESULT  res;
TCHAR    lfName[_MAX_LFN];

static uint8_t tmpCmd[maxCmdLenght];
static uint8_t lenghtCmd=maxCmdLenght;
static uint8_t *pCmd=tmpCmd;

extern volatile t_fifobuf  uart2RxFifo;
extern uint32_t Volume;

void exeCmd (uint8_t *pBuf);
void reqAnsw(uint8_t *pBuf);
//---------------------------------------------------
void cmdRSprocess (void){
   uint8_t tmp;
   
   while(fifoGetCountDataMacro(&uart2RxFifo)){
      fifoRestoreData(&uart2RxFifo,&tmp);
//      if(tmp>='a') tmp-=('a'-'A');
      if((tmp=='\r')||(--lenghtCmd==0)){
         if(tmp=='\r'){
            *pCmd++=0;
            exeCmd(tmpCmd);
         }else{
            printf(ansInvCmd); printf("\r\n");   
         }
         lenghtCmd=maxCmdLenght;      
         pCmd=tmpCmd;
      }else *pCmd++=tmp;
   }
}
//==================================================
void exeCmd(uint8_t *pBuf){
   uint8_t tmp;
  
   if(*pBuf=='\n') pBuf++;
   if(*pBuf=='?'){
      reqAnsw(pBuf);
      return;
   }
   //Volume Set
   if(strncmp((const char *)pBuf,cVolSet)==0){
      pBuf+=4;
      tmp=(*pBuf++)-'0';
      tmp*=10;
      tmp+=(*pBuf++)-'0';
      if(tmp<=maxVolume){
         Volume=tmp;
         printf(ansVol); printf("%02d\r\n",Volume);
      }else{
         printf(ansInvPar); printf("\r\n");
      }
      return;
   }
   //Play
   if(strncmp((const char *)pBuf,cPlay)==0){
      pBuf+=5;
      wavePlay(pBuf);
      return;
   }
   printf(ansInvCmd); printf("\r\n");
}
//-----------------------------------------------
void reqAnsw(uint8_t *pBuf){

   //Volume
   if(strncmp((const char *)pBuf,qVol)==0){
      printf(ansVol); printf("%02d\r\n",Volume);
      return;
   }
   //Dir
   if(strncmp((const char *)pBuf,qDir)==0){
      fno.lfname=lfName;
      fno.lfsize = sizeof lfName;
      res=f_opendir(&dir,"0:/");
      while(res==FR_OK){
         res=f_readdir(&dir, &fno);
         if((res!=FR_OK||fno.fname[0]==0)) break;
         if(*fno.lfname){
            printf("%s\r\n",fno.lfname);
         }else{
            printf("%s\r\n",fno.fname);
         }
      }
      return;
   }
   //Help
   if(strncmp((const char *)pBuf,qHelp)==0){
      printf("%s",ansHelp);
      return;
   }
   printf(ansInvCmd); printf("\r\n");
}
