/**
* \file fifobuf.h
* \brief Plik naglowkowy do "fifobuf.c".
* \author Tadeusz Sondej
* \date 12-09-2009
*/

#ifndef __FIFOBUF_H__
#define __FIFOBUF_H__


#include "sysfun.h"

typedef           int32_t  t_result;
typedef volatile  uint8_t  t_fifodata;

typedef volatile struct{
  uint32_t    SizeBuf;  
  uint32_t    Count;    
  t_fifodata  *pBegin;
  t_fifodata  *pEnd;
  t_fifodata  *pFirstIn;
  t_fifodata  *pFirstOut;
  uint32_t    State;
} t_fifobuf;

#define ERR_NO_ERROR            0

#define ERR_FIFO_OVERFLOW      (-40)
#define ERR_FIFO_EMPTY         (-41)
#define ERR_FIFO_UNKNOWN_PARAM (-42)
#define ERR_FIFO_PTR_NULL      (-43)


t_result fifoInitialize( t_fifobuf *cbuf,
                         t_fifodata *begin,
                         t_fifodata *end, uint16_t size );
/*-------------------------------------------------------------------------
  Inicjuje bufor FIFO.
  Oba paramertry "begin" i "end" okrelaj konkretne adresy
  i s one przepisywane odpowiednio do pl "pBegin" i "pEnd".
  Parametr size okrela rozmiar bufora w danych t_fifodata.
  Zwraca warto:
      - ErrNoError - jeli si powioda,
      - erNoMemory - brak pamici,
      - ErrUnknownParametr - niewaciwe wartoci parametrw.
-------------------------------------------------------------------------*/

t_result fifoRestoreData( t_fifobuf *cbuf, t_fifodata *dana );
/*-------------------------------------------------------------------------
  Odczytuje dan z bufora okrelonego parametrem "cbuf".
  Odczytan dan umieszcza w miejscu okrelonym parametrem "data"
  Funkcja zwraca wartoci:
      - ErrNoError - jeli si powioda,
      - ErrFifoEmpty - bufor FIFO jest pusty (odczyt danej niemoliwy).
-------------------------------------------------------------------------*/


#define fifoSaveDataFast( cbuf, dana ) \
  {  \
   if((cbuf)->Count<(cbuf)->SizeBuf)\
  {                              \
    *((cbuf)->pFirstIn)=dana;       \
    if((cbuf)->pFirstIn==(cbuf)->pEnd) \
      (cbuf)->pFirstIn=(cbuf)->pBegin;  \
    else                             \
      (cbuf)->pFirstIn++;               \
    (cbuf)->Count++;                    \
  }  \
    }

#define fifoGetCountDataMacro( cbuf ) ((cbuf)->Count)
#define fifoGetFreeSpace( cbuf ) ((cbuf)->SizeBuf-(cbuf)->Count)

#endif // __FIFOBUF_H__

