/**
* \file player.c
* \brief Plik z funkcjami generowania muzyki
* \author Marek Sawicki
* \date 18-07-2012
*/
//-------------------------------------------------------------------------------------------------
#include "player.h"



volatile t_fifodata playerBuf[playerBuf_SIZE];
volatile t_fifobuf  playerFifo;

static   uint32_t sigFlag;   //0-unisgned, 1-signed
static   uint32_t lenSample; //1,2,3 (w bajtach)
static   uint32_t lSek; //dla mono 0, dla stereo 1*lenSample
/*---------------------------------------------
   Inicjalizacja peryferiw dla generatorw
----------------------------------------------*/
void playerInit(void){
   GPIO_InitTypeDef  GPIO_InitStructure;
   TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;
   TIM_OCInitTypeDef  TIM_OCInitStructure;
   
   //IO
   RCC_APB2PeriphClockCmd(outMSB_GPIO_CLK, ENABLE); // Enable the Clock 
   GPIO_InitStructure.GPIO_Pin = outMSB_GPIO_PIN | outLSB_GPIO_PIN;
   GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP; 
   GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
   GPIO_Init(outMSB_GPIO_PORT, &GPIO_InitStructure);
   
   //TIM2 load Sample
   RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE);
   TIM_TimeBaseStructure.TIM_Period = 0; 
   TIM_TimeBaseStructure.TIM_Prescaler = 0;
   TIM_TimeBaseStructure.TIM_ClockDivision = 0;
   TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Down;
   TIM_TimeBaseInit(TIM2, &TIM_TimeBaseStructure);
   TIM_ITConfig(TIM2, TIM_IT_Update, ENABLE);

   //TIM3 PWM generator
   RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3, ENABLE);
   TIM_TimeBaseStructure.TIM_Period = 255; //8 bit PWM
   TIM_TimeBaseStructure.TIM_Prescaler = 0;
   TIM_TimeBaseStructure.TIM_ClockDivision = 0;
   TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Down;
   TIM_TimeBaseInit(TIM3, &TIM_TimeBaseStructure);
   //kanal zgrubny
   TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM1;
   TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
   TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;
   TIM_OCInitStructure.TIM_Pulse = 255;         
   TIM_OC1Init(TIM3, &TIM_OCInitStructure);   
   //kanal dokladny
   TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM1;
   TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
   TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;
   TIM_OCInitStructure.TIM_Pulse = 255;         
   TIM_OC2Init(TIM3, &TIM_OCInitStructure);   
   
   fifoInitialize(&playerFifo,&playerBuf[0],
                 &(playerBuf[playerBuf_SIZE-1]),playerBuf_SIZE);
}
//========================================================
void playerStart(t_playerStruct *playerStruct){
   
   TIM2->ARR=timFreq/playerStruct->fSample;
   lenSample=playerStruct->lSample/8;
   if(lenSample==1) sigFlag=0; else sigFlag=1;
   lSek=(playerStruct->lChanel-1)*lenSample;
   
   TIM_Cmd(TIM3, ENABLE);
   TIM_Cmd(TIM2, ENABLE);
   AmpOn();
}
//========================================================
void playerStop(void){

   TIM_Cmd(TIM2, DISABLE);
   TIM_Cmd(TIM3, DISABLE);
   AmpOff();

}
//========================================================
__irq void TIM2_IRQHandler (void){
   uint16_t  tmp,i;
   uint8_t  *pTmp=(uint8_t *)&tmp;
   t_result status;

   TIM_ClearITPendingBit(TIM2, TIM_IT_Update);
   for(i=lenSample;i>0;i--){
      status=fifoRestoreData(&playerFifo, pTmp++);   
   }
   if(status!=ERR_NO_ERROR){
      playerStop();
      return;
   }
   if(sigFlag) tmp+=32768;
   tmp/=10;
   TIM3->CCR1=tmp>>8;
   TIM3->CCR2=tmp&0x00FF;
   if(lSek){
      for(i=lSek;i>0;i--){
         fifoRestoreData(&playerFifo, pTmp);   
      }   
   }
}
