/**
* \file uart.c
* \brief Plik z funkcjami obslugi interfejsu UART.
* \author Marek Sawicki
* \date 22-07-2011
*/

#include <stdio.h>
#include "stm32f10x.h"
#include "uart.h"

volatile t_fifodata uart2RxBuf[UART2_SIZE_RXBUF_FIFO];
volatile t_fifobuf  uart2RxFifo;


void uartsInit (void){
   USART_InitTypeDef  USART_InitStructure;
   GPIO_InitTypeDef GPIO_InitStructure;

//----------- uart 2 -----------------------------------------------  
   RCC_APB2PeriphClockCmd(UART2_GPIO_CLK, ENABLE); // Enable GPIO clock 
   RCC_APB1PeriphClockCmd(UART2_CLK, ENABLE); // Enable UART clock   

   // Configure USART Tx, RTS as alternate function push-pull
   GPIO_InitStructure.GPIO_Pin = UART2_TxPin;
   GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
   GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
   GPIO_Init(UART2_GPIO, &GPIO_InitStructure);
   // Configure USART Rx, CTS as input Pull Up 
   GPIO_InitStructure.GPIO_Pin = UART2_RxPin;
   GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
   GPIO_Init(UART2_GPIO, &GPIO_InitStructure);
   // Configure USART
   USART_InitStructure.USART_BaudRate = 115200;    // max. 4500000 dla UART 1 
   USART_InitStructure.USART_WordLength = USART_WordLength_8b;
   USART_InitStructure.USART_StopBits = USART_StopBits_1;
   USART_InitStructure.USART_Parity = USART_Parity_No;
   USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
   USART_InitStructure.USART_Mode = USART_Mode_Tx |USART_Mode_Rx;
   USART_Init(USART2, &USART_InitStructure); 

   fifoInitialize(&uart2RxFifo,&uart2RxBuf[0],
                 &(uart2RxBuf[UART2_SIZE_RXBUF_FIFO-1]),UART2_SIZE_RXBUF_FIFO);

   USART_ITConfig(USART2, USART_IT_RXNE, ENABLE); // Enable USART interrupts
   USART_Cmd(USART2, ENABLE); // Enable the USART 
}
//===============================================================================
//  USART2_IRQHandler Handles global interrupt request.
__irq void USART2_IRQHandler (void){
   volatile uint32_t status;
   uint8_t rxb;

   status = USART2->SR;
   if(status&0x0F){ // tylko flagi bledu
      rxb=USART2->DR; // dla skasowania bitow bledu
      // Bit 3 ORE: Overrun error, It is cleared by a software sequence (an read to the
      // USART_SR register followed by a read to the USART_DR register
      // Bit 2 NE: Noise error flag, clear as ORE
      // Bit 1 FE: Framing error, clear as ORE
      // Bit 0 PE: Parity error, clear as ORE, The software must wait for the RXNE flag to be set before clearing the PE bit.
      status = USART2->SR; // ponowne czytanie dla skasowania flag bledow
   }else{
      if(status & USART_FLAG_RXNE){       // read interrupt
         USART2->SR &= ~USART_FLAG_RXNE;   // clear interrupt
         rxb=(USART2->DR & 0xFF);
         fifoSaveDataFast(&uart2RxFifo,rxb);
         //uart2WriteByte(rxb);
      }else if(status & USART_FLAG_TXE){
         USART2->SR &= ~USART_FLAG_TXE;  // clear interrupt
      }
   }
}

/*----------------------------------------------------------------------------
  Write character to Serial Port
 *----------------------------------------------------------------------------*/
int sendchar (int c){
   
   while (!(USART2->SR & USART_FLAG_TXE));
   USART2->DR = (c & 0xFF);
   return (c);
}

/*----------------------------------------------------------------------------
  Read character from Serial Port   
 *----------------------------------------------------------------------------*/
int getkey (void){
   
   return (0);
}
